# -*- coding: utf-8 -*-
def www_smhi_file(sStationID, sDataPeriod, sTargetPath, sStartYear, sStartMonth, sOS):
    ########### This script just downloads data from SMHI and saves to your computer. Read below about how the encoding and character sets are handled in this file. #################
    # UTF-8 and Unicode cannot be compared. Actually, comparing UTF-8 and Unicode is like comparing apples and oranges.
    # UTF-8 is an encoding used to translate binary data (bytes) into numbers. Unicode is a character set used to translate numbers into characters.
    #
    # The SMHI files are read as bytes and directly converted from UTF-8 bytes to Unicode-text with this line:
    # sL = sLbytes.decode('utf-8-sig') # The word .decode specifies that we want to convert bytes to text.
    # The SMHI files are encoded as 'utf-8 with Bom' and therefore read as 'utf-8-sig' in both Python2 and 3 instead of just 'utf-8'.
    # Strings created in Python3 are by default 'utf-8' so therefore variables with data read from SMHI files can easily be joined together as 'utf-8'.
    # Strings created in Python2 are by default 'ascii' but if you in Python2 join an 'ascii'-string with a variable with 'utf-8' encoded Unicode content then the resulting string is automatically 'utf-8' Unicode. Therefore the code can be the same for Python2 and 3.
    # The data read from SMHI is saved to your computer as files (YYYY-MM.csv). It is written as bytes 'wb' and encoded as 'utf-8' both in Python2 and 3. This is handled automatically with 'codecs.open(...' below.
    #
    # The result is sent back to the Basic code as a string from the variablename - line. If everything went ok this string will start with 1. If an error occured the string will start with a higher number than 1.
    # To find out more about which encodings that are used you can uncomment the lines at the bottom of this file and run the script.
    #
    # Read comment below to see how the result was sent back to Basic before before it was changed in the final Python script:
    # For some reason Python3 needs to send the string back as text so therefore it's not encodede with .encode.
    # Python2 on the other hand needs to send it as bytes to not create an error. Therefore it's encoded to bytes. The bytes also need to be encoded with the right format so Swedish characters are shown correct in the Basic code and the right encoding depends on the Operating system's default encoding. This is found out with: sys.getfilesystemencoding()
    ############################################ ############################################ ############################################ ############################################
    try:
        sStationID = str(sStationID) #Needs to be converted to str because if an error occurs in Python2 and the varialbe sError i used, then an encoding error would occur instead which shows error 7 instead of the real error.
        sDataPeriod = str(sDataPeriod) #See comment above
        sUrl = ''.join(('http://opendata-download-metobs.smhi.se/api/version/1.0/parameter/2/station/', sStationID, '/period/', sDataPeriod, '/data.csv'))
        #sUrl = ''.join(('http://localhost/senib-smhi-test-folder/', sStationID, '/period/', sDataPeriod, '/data.csv')) # uncomment for testing only
        sError = ''.join(('Klimatnummer: ', sStationID, ', Dataperiod: ', sDataPeriod, ' >>>\n'))
        iStartYear = int(sStartYear) # Startyear to get statistics for
        iStartMonth = int(sStartMonth) # Will be set to 4 months before today's month in Basic if file with latest-months is read from SMHI so corrected-archive files are not overwritten
        iLc = 0 # integer, Line Counter of which line is read in csv file from SMHI
        iDc = 0 # integer, Downloaded Date Counter of number of dates with temperatures written to the temperature Files (YYYY-MM.csv)
        iPrevY = 0 # integer, Previous Year
        iPrevM = 0 # integer, Previous Month
        bFileOpen = False # boolean, True if a file is open (YYYY-MM.csv)
        # line = '' # line to return to Basic
        # sLbytes = ''# string, Line with bytes, read from SMHI temperature file
        # sL = '' # string, Line with Unicode utf-8 text converted from variable sLbytes
        # sStation = '' # Station ID read from line 2 of a SMHI temperature file
        # sDataType = '' # Type of data read at line 5 of a SMHI temperature file
        # sY = '' # string, Year
        # sM = '' # string, Month
        # sD = '' # string, Day
        # iY = 0 # integer, Year
        # iM = 0 # integer, Month
        # iD = 0 # integer, Day
        # iSemi1 = 0 # integer, Position of Semicolon 1 in sL
        # iSemi2 = 0 # integer, Position of Semicolon 2 in sL
        # iSemi4 = 0 # integer, Position of Semicolon 4 in sL
        import sys
        import os
        from os import path
        if sys.hexversion >= 0x030000F0:
            runningPython3 = True
        else:
            runningPython3 = False
        if runningPython3 == True: #--------------- Running Python3 --------------------
            import urllib.request, urllib.error
            import socket
            import codecs
            from urllib.request import urlopen
            from urllib.error import URLError
            try:
                response = urllib.request.urlopen(sUrl, None, 2.5) #(Open Url, get/post data = None, timeout in sec on url request)
                if response.getcode() == 200: #Confirm that the file exist. Otherwise the contents of an 404 page might be downloaded.
                    if not os.path.exists(sTargetPath): #Check if target directory exists
                        os.makedirs(sTargetPath) #Create target directory 
                    for sLbytes in response: #Read the file
                        #¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤ Start same code for Python2 ¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤
                        sL = sLbytes.decode('utf-8-sig') #Convert bytes to unicode. SMHI file is utf-8 with Bom therefore utf-8-sig and not utf-8
                        iLc += 1 #Count which line in read
                        if iLc == 2: #If line 2 is read retrieve Station and StationID from line
                            if sL.count(';') >= 2: #Count number of ';' in string. More than or eaqual to 2 do this.
                                iSemi1 = sL.find(';', 0) #find first ';' in string
                                iSemi2 = sL.find(';', iSemi1+1) #find second ';' in string
                                sStation = ''.join((sL[0:iSemi1], ';', sL[iSemi1+1:iSemi2])) #Join the strings Station and Station ID
                            else: #Line 2 didn't contain 2 or more ';' so the line does not show Station and Station ID
                                sStation = ''.join(('Stationsnamn - unknown', ';', 'Klimatnummer - unknown')) #Continue reading SMHI file and insert this at the top
                        if iLc == 5: #If reading line 5 in SMHI file
                            sDataType = sL[0:32] #Retrive which type of data that is stored in the file
                        if iLc >= 11 and len(sL) >= 52 and sL.count(';') >= 4: #Don't process lines earlier than 11 because they might contain example data, Line must be more than 51 characters long and have more than 3 ';'
                            iSemi4 = sL.find(';', 51) #Find the position of ';' that comes right after the temperature
                            if sL[50:51] == ';' and iSemi4 >= 52: #It's only a valid temperature if the semi-colons are in the right place
                                sY = sL[0:4] #Get Year
                                sM = sL[5:7] #Get Month
                                sD = sL[8:10] #Get Day
                                try:
                                    iY = int(sY) #Test if Year is 4 numbers
                                    iM = int(sM) #Test if Month is 2 numbers
                                    iD = int(sD) #Test if Day is 2 numbers
                                except ValueError:
                                    continue #They were not all numbers so continue to read next line in SMHI file
                                else:
                                    if iY >= iStartYear and iY <= 2200 and iM >= 1 and iM <= 12 and iD >= 1 and iD <= 31: #Check that years and months are in the requested range
                                        if iY == iStartYear and iM < iStartMonth: #If the read year and month is earlier than requested don't process it
                                            d = 'dummy' #Just continue with next line in SMHI file
                                        else:
                                            if iY > iPrevY or iM > iPrevM: #If it's a new month then create a new temperature file
                                                if bFileOpen == True: #Is old file flagged open?
                                                    file.close() #Close the old file
                                                    bFileOpen = False #Flag that the file is closed
                                                file = codecs.open(''.join((sTargetPath, sY, '-', sM, '.csv')), 'wb', 'utf-8') #Open file for writing, 'wb' = write bytes, 'w' = write. Use utf-8 without Bom. Any existing file is overwritten.
                                                bFileOpen = True #Flag that a file is open
                                                file.write(''.join((sStation, '\n', sDataType))) #Write Station, Station ID and DataType to the two top lines of the file
                                            file.write(''.join(('\n', sL[0:10], ';', sL[51:iSemi4]))) #Write date and temperature to file
                                            iDc += 1 #Counter for how dates with temperatures are written to disk
                                    iPrevY = iY #iPreviousYear will be used in the next loop to see if it's a new month
                                    iPrevM = iM
                if bFileOpen == True: #If a file is flagged open then close it
                    file.close()
                    #¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤ End same code for Python2 ¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤
            except urllib.error.URLError as e:
                if hasattr(e, 'reason'):
                    line = '2, ' + sError + 'Python3 script failed to reach the server. Reason: ' + str(e.reason)
                elif hasattr(e, 'code'):
                    line = '3, ' + sError + 'Python3 script couldn\'t make the server fulfill the request. Error code: ' + str(e.code)
            except socket.timeout:
                line = '4, Python3 script timed out when trying to reach the server!'
            except: #catch *all* inner exceptions
                e = sys.exc_info()[0]
                line = ( '5, ' + sError + 'Python3 script error in file script_smhi.py. Error is: %s' % e )
            else:
                response.close()
                try:
                    #line = ''.join(('1;', sStation, ';CreatedFiles;', str(iDc), ';', sDataType)) #Old join string to return to Basic code.
                    line = ''.join(('1;', str(iDc))) #Return number of dates downloaded from Internet
                except: #catch encoding errors in string that is going to be sent back to Basic code
                    line = '6, ' + sError + 'Python3 encoding error on string that is sent back to Basic code. The import of data from SMHI seems to have worked fine though.'
                else:
                    d = 'dummy'
        else: #--------------- Running Python2 --------------------
            import urllib2
            import socket
            import codecs
            from urllib2 import urlopen, URLError
            try:
                response = urllib2.urlopen(sUrl, None, 2.5) #(Open Url, get/post data = None, timeout in sec on url request)
                if response.getcode() == 200: #Confirm that the file exist. Otherwise the contents of an 404 page might be downloaded.
                    if not os.path.exists(sTargetPath): #Check if target directory exists
                        os.makedirs(sTargetPath) #Create target directory 
                    for sLbytes in response: #Read the file
                        #¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤ Start same code for Python3 ¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤
                        sL = sLbytes.decode('utf-8-sig') #Convert bytes to unicode. SMHI file is utf-8 with Bom therefore utf-8-sig and not utf-8
                        iLc += 1 #Count which line in read
                        if iLc == 2: #If line 2 is read retrieve Station and StationID from line
                            if sL.count(';') >= 2: #Count number of ';' in string. More than or eaqual to 2 do this.
                                iSemi1 = sL.find(';', 0) #find first ';' in string
                                iSemi2 = sL.find(';', iSemi1+1) #find second ';' in string
                                sStation = ''.join((sL[0:iSemi1], ';', sL[iSemi1+1:iSemi2])) #Join the strings Station and Station ID
                            else: #Line 2 didn't contain 2 or more ';' so the line does not show Station and Station ID
                                sStation = ''.join(('Stationsnamn - unknown', ';', 'Klimatnummer - unknown')) #Continue reading SMHI file and insert this at the top
                        if iLc == 5: #If reading line 5 in SMHI file
                            sDataType = sL[0:32] #Retrive which type of data that is stored in the file
                        if iLc >= 11 and len(sL) >= 52 and sL.count(';') >= 4: #Don't process lines earlier than 11 because they might contain example data, Line must be more than 51 characters long and have more than 3 ';'
                            iSemi4 = sL.find(';', 51) #Find the position of ';' that comes right after the temperature
                            if sL[50:51] == ';' and iSemi4 >= 52: #It's only a valid temperature if the semi-colons are in the right place
                                sY = sL[0:4] #Get Year
                                sM = sL[5:7] #Get Month
                                sD = sL[8:10] #Get Day
                                try:
                                    iY = int(sY) #Test if Year is 4 numbers
                                    iM = int(sM) #Test if Month is 2 numbers
                                    iD = int(sD) #Test if Day is 2 numbers
                                except ValueError:
                                    continue #They were not all numbers so continue to read next line in SMHI file
                                else:
                                    if iY >= iStartYear and iY <= 2200 and iM >= 1 and iM <= 12 and iD >= 1 and iD <= 31: #Check that years and months are in the requested range
                                        if iY == iStartYear and iM < iStartMonth: #If the read year and month is earlier than requested don't process it
                                            d = 'dummy' #Just continue with next line in SMHI file
                                        else:
                                            if iY > iPrevY or iM > iPrevM: #If it's a new month then create a new temperature file
                                                if bFileOpen == True: #Is old file flagged open?
                                                    file.close() #Close the old file
                                                    bFileOpen = False #Flag that the file is closed
                                                file = codecs.open(''.join((sTargetPath, sY, '-', sM, '.csv')), 'wb', 'utf-8') #Open file for writing, 'wb' = write bytes, 'w' = write. Use utf-8 without Bom. Any existing file is overwritten.
                                                bFileOpen = True #Flag that a file is open
                                                file.write(''.join((sStation, '\n', sDataType))) #Write Station, Station ID and DataType to the two top lines of the file
                                            file.write(''.join(('\n', sL[0:10], ';', sL[51:iSemi4]))) #Write date and temperature to file
                                            iDc += 1 #Counter for how dates with temperatures are written to disk
                                    iPrevY = iY #iPreviousYear will be used in the next loop to see if it's a new month
                                    iPrevM = iM
                if bFileOpen == True: #If a file is flagged open then close it
                    file.close()
                    #¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤ End same code for Python3 ¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤¤
            except URLError as e:
                if hasattr(e, 'reason'):
                    line = '2, ' + sError + 'Python2 script failed to reach the server. Reason: ' + str(e.reason)
                elif hasattr(e, 'code'):
                    line = '3, ' + sError + 'Python2 script couldn\'t make the server fulfill the request. Error code: ' + str(e.code)
            except socket.timeout:
                line = '4, ' + sError + 'Python2 script timed out when trying to reach the server!'
            except: #catch *all* inner exceptions
                e = sys.exc_info()[0]
                line = ( '5, ' + sError + 'Python2 script error in file script_smhi.py. Error is: %s' % e )
            else:
                response.close()
                try:
                    #sEncoding = sys.getfilesystemencoding() #Used for old string. Get default decoding for the operating sytem. On Mac OS X, the encoding is 'utf-8'. Windows is 'mbcs'.
                    #sStation = sStation.encode(sEncoding) #Used for old string. Convert Unicode to bytes before returning string to Basic
                    #sDataType = sDataType.encode(sEncoding) #Used for old string.
                    #line = ''.join((b'1;', sStation, b';CreatedFiles;', str(iDc), b';', sDataType)) #Old join string to return to Basic code. b infront of the 'strings' converts to bytes.
                    line = ''.join((b'1;', str(iDc))) #Return number of dates downloaded from Internet
                except: #catch encoding errors in string that is going to be sent back to Basic code
                    line = '6, ' + sError + 'Python2 encoding error on string that is sent back to Basic code. The import of data from SMHI seems to have worked fine though.'
                else:
                    d = 'dummy'
    except: #catch *all* outer exceptions
        e = sys.exc_info()[0]
        line = ( '7, ' + sError + 'Python2 or Python3 script error in file script_smhi.py. Error is: %s' % e )
    else:
        d = 'dummy'
        # line = sys.getdefaultencoding() # Uncomment to see default string encoding used by the Unicode implementation ('ascii' in Python2 and 'utf-8' in Python3) - https://docs.python.org/2/library/sys.html
        # line = sys.getfilesystemencoding() # Uncomment to see default encoding of operating system. 'utf-8' on Mac OS X, 'mbcs' on Windows.
    return line
